#include <HRlib.h>
#include <config.h>
#include <WiFi.h>
#include <PubSubClient.h>

//OLED
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

WiFiClient wifiClient;
PubSubClient mqtt(MQTT_BROKER, 1883, wifiClient);
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

enum task1_state_t {
  BUZZER_ON,
  BUZZER_OFF,
};

enum task2_state_t {
  HR_ON,
  HR_OFF,
};

enum task3_state_t {
  BEATAVG_ON,
  BEATAVG_OFF,
};

//set state variable
task1_state_t task1_state;
task2_state_t task2_state;
task3_state_t task3_state;

uint32_t timestamp1, timestamp2, timestamp3;

void connect_wifi() {
  printf("WiFi MAC address is %s\n", WiFi.macAddress().c_str());
  printf("Connecting to WiFi %s.\n", WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    printf(".");
    fflush(stdout);
    delay(500);
  }
  printf("\nWiFi connected.\n");
}

void connect_mqtt() {
  printf("Connecting to MQTT broker at %s.\n", MQTT_BROKER);
  if (!mqtt.connect("", MQTT_USER, MQTT_PASS)) {
    printf("Failed to connect to MQTT broker.\n");
    for (;;) {} // wait here forever
  }
  printf("MQTT broker connected.\n");
}

void setup() {
  max30102_setup();
  Serial.begin(115200);
  timestamp1 = millis();
  timestamp2 = millis();
  timestamp3 = millis();

  //initialize state
  task1_state = BUZZER_ON;
  task2_state = HR_ON;
  task3_state = BEATAVG_ON;

   //connect to node red
  connect_wifi();
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println("");
  connect_mqtt();

   //check display
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
      Serial.println(F("SSD1306 allocation failed"));
      for (;;);
    }
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0, 10);
}

void task2() {
  uint32_t now;
  //IR red change to heart rate
  if (task2_state == HR_ON) {
    now = millis();
    if (now - timestamp2 >= 1) {
      long irValue = particleSensor.getIR();
      if (checkForBeat(irValue) == true)
      {
        //We sensed a beat!
        long delta = millis() - lastBeat;
        lastBeat = millis();
        beatsPerMinute = 60 / (delta / 1000.0);
        if (beatsPerMinute < 255 && beatsPerMinute > 20 && irValue >= 50000)
        {
          rates[rateSpot++] = (byte)beatsPerMinute; //Store this reading in the array
          rateSpot %= RATE_SIZE; //Wrap variable
          //Take average of readings
          beatAvg = 0;
          for (byte x = 0 ; x < RATE_SIZE ; x++)
            beatAvg += rates[x];
          beatAvg /= RATE_SIZE;
        }
      }
      timestamp2 = now;
    }
  }
}

void task3() {
  uint32_t now;
  //publish beatAvg
  if (task3_state == BEATAVG_ON) {
    now = millis();
    if (now - timestamp3 >= 1500) {
      Serial.print("beatAvg: ");
      Serial.println(beatAvg);

      //display
      display.clearDisplay();
      display.setCursor(60, 15);
      display.println("BPM");
      display.setCursor(60, 30);
      display.println(beatAvg);
      display.display();

      //node red
      String beatAvgStr = String(beatAvg);
      mqtt.publish(TOPIC_BEATAVG,  beatAvgStr.c_str());
      timestamp3 = now;
    }
  }
}

void task1() {
  //buzzer
  uint32_t now;
  if (task1_state == BUZZER_ON) {
    now = millis();
    if (beatAvg >= 20 && now - timestamp1 >= (60/beatAvg)*1000) {
      //tone();
      tone(14,150);
      task1_state = BUZZER_OFF;
      timestamp1 = now;
    }
  }
  else if (task1_state == BUZZER_OFF) {
    now = millis();
    if (now - timestamp1 >= 100) {
      noTone(14);
      task1_state = BUZZER_ON;
      timestamp1 = now;
    }
  }
}

void loop() {
  task1();
  task2();
  task3();
}
