#include <WiFi.h>
#include <PubSubClient.h>
#include <config.h>

//Audio
#include <Audio.h>

//OLED
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Wire.h>

WiFiClient wifiClient;
PubSubClient mqtt(MQTT_BROKER, 1883, wifiClient);
Audio audio;
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

//constant variable
const char* weight[23]
  = { "Chest Press","Break","Arm Curl","Break","Triceps Kickback","Break",
  "Bent-over Row","Break","Plank","Break","Reverse Crunch","Break","Lateral Twist",
  "Break","Crunch","Break","Good Morning","Break","Lunge","Break","Squat","Break","Lateral Lunge" };
const int weight_size = sizeof(weight)/sizeof(weight[0]);
const char* CONGRATULATION = "Congratulation";
const char* BREAK = "Break";
const char* SPACE = " ";
const char* CHEST_PRESS = "Chest Press";
const char* lang = "en-US";
const int t = 25;
const int s = 10;

void connect_wifi() {
  printf("WiFi MAC address is %s\n", WiFi.macAddress().c_str());
  printf("Connecting to WiFi %s.\n", WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    printf(".");
    fflush(stdout);
    delay(500);
  }
  printf("\nWiFi connected.\n");
}

void connect_mqtt() {
  printf("Connecting to MQTT broker at %s.\n", MQTT_BROKER);
  if (!mqtt.connect("", MQTT_USER, MQTT_PASS)) {
    printf("Failed to connect to MQTT broker.\n");
    for (;;) {} // wait here forever
  }
  printf("MQTT broker connected.\n");
}

void stop(int x) {
  for(int i = x; i > -1; i--) {
    String payload(i);
    //display
    display.clearDisplay();
    display.setCursor(0, 0);
    display.println("Break:");
    display.setCursor(60, 30);
    display.println(i);
    display.display(); 

    mqtt.publish(TOPIC_DISPLAY_WORKOUT, BREAK);
    mqtt.publish(TOPIC_DISPLAY_TIMER, payload.c_str());
    delay(1000);
  }
}

void timer(int x, int i) {
  for(int t = x; t > -1; t--) {
    String payload(t);
    //display
    display.clearDisplay();
    display.setCursor(0, 0);
    display.println("Workout:");
    display.setCursor(60, 0);
    display.println(((i/2)+1));
    display.setCursor(15, 15);
    display.println(weight[i]);
    display.setCursor(60, 30);
    display.println(t);
    display.display(); 

    mqtt.publish(TOPIC_DISPLAY_TIMER, payload.c_str());
    delay(1000);
  }
}

void workout(int i) {
  String payload(weight[i]);
  mqtt.publish(TOPIC_DISPLAY_WORKOUT, payload.c_str());
  if (i <= 6) {
      if (i==0) {
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if (i==1) { 
        audio.connecttospeech("Arm Curl", lang);
        stop(s); 
      }
      if (i==2) {
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if (i==3) { 
        audio.connecttospeech("Triceps Kickback", lang);
        stop(s); 
      }
      if (i==4) { 
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if (i==5) { 
        audio.connecttospeech("Bent-over Row", lang);
        stop(s); 
      }
      if (i==6) { 
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      digitalWrite(RED_GPIO, HIGH);
  } else if (i > 6 && i < 15) {
      if (i==7) { 
        audio.connecttospeech("Plank", lang);
        stop(s); 
      }
      if (i==8) { 
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if (i==9) { 
        audio.connecttospeech("Reverse Crunch", lang);
        stop(s); 
      }
      if (i==10) {
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if (i==11) { 
        audio.connecttospeech("Lateral Twist", lang);
        stop(s);
      }
      if (i==12) {
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if(i==13) {
        audio.connecttospeech("Crunch", lang);
        stop(s);
      }
      if(i==14) {
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      digitalWrite(RED_GPIO,0);
      digitalWrite(YELLOW_GPIO,1);
  } else {
      if(i==15) {
        audio.connecttospeech("Good Morning", lang);
        stop(s);
      }
      if(i==16) {
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if(i==17) {
        audio.connecttospeech("lunge", lang);
        stop(s);
      }
      if(i==18) {
        audio.connecttospeech("Break",lang);
        timer(t, i);
      }
      if(i==19) {
        audio.connecttospeech("Squat", lang);
        stop(s);
      }
      if(i==20) {
        audio.connecttospeech("Break", lang);
        timer(t, i);
      }
      if(i==21) {
        audio.connecttospeech("Lateral Lunge", lang);
        stop(s);
      }
      if(i==22) {
        audio.connecttospeech("Congratulation", lang);
        timer(t, i);
        
      }
      digitalWrite(YELLOW_GPIO,0);
      digitalWrite(GREEN_GPIO,1);
  }
  
}

void setup() {
  Serial.begin(115200);
  //set led pin
  pinMode(RED_GPIO, OUTPUT);
  pinMode(YELLOW_GPIO, OUTPUT);
  pinMode(GREEN_GPIO, OUTPUT);

  //turn led off
  digitalWrite(RED_GPIO,0);
  digitalWrite(YELLOW_GPIO,0);
  digitalWrite(GREEN_GPIO,0);

  //SDA => GPIO48, SCL => GPIO47
  Wire.begin(48, 47); 

  //connect to node red
  connect_wifi();
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println("");
  connect_mqtt();

  // Connect MAX98357a I2S Amplifier Module
  audio.setPinout(I2S_BCLK, I2S_LRC, I2S_DOUT);
  
  // Set thevolume (0-100)
  audio.setVolume(50);

  //start with chest press
  mqtt.publish(TOPIC_DISPLAY_WORKOUT, CHEST_PRESS);
  audio.connecttospeech("Chest Press", lang);
  digitalWrite(RED_GPIO, HIGH);

  //check display
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
      Serial.println(F("SSD1306 allocation failed"));
      for (;;);
    }
  display.setTextSize(1);
  display.setTextColor(WHITE);
}

void loop() {
  // Run audio player
  audio.loop();
}

// Audio status functions
 
void audio_info(const char *info) {
  Serial.print("info        "); Serial.println(info);
}
void audio_id3data(const char *info) { //id3 metadata
  Serial.print("id3data     "); Serial.println(info);
}
void audio_eof_mp3(const char *info) { //end of file
  Serial.print("eof_mp3     "); Serial.println(info);
}
void audio_showstation(const char *info) {
  Serial.print("station     "); Serial.println(info);
}
void audio_showstreaminfo(const char *info) {
  Serial.print("streaminfo  "); Serial.println(info);
}
void audio_showstreamtitle(const char *info) {
  Serial.print("streamtitle "); Serial.println(info);
}
void audio_bitrate(const char *info) {
  Serial.print("bitrate     "); Serial.println(info);
}
void audio_commercial(const char *info) { //duration in sec
  Serial.print("commercial  "); Serial.println(info);
}
void audio_icyurl(const char *info) { //homepage
  Serial.print("icyurl      "); Serial.println(info);
}
void audio_lasthost(const char *info) { //stream URL played
  Serial.print("lasthost    "); Serial.println(info);
}

void audio_eof_speech(const char *info) {
  Serial.print("eof_speech  "); Serial.println(info);
  static int i=0;
  const char* lang = "en-US";
  workout(i);
  i++;
  if(i==23) {
    mqtt.publish(TOPIC_DISPLAY_WORKOUT, CONGRATULATION);
    mqtt.publish(TOPIC_DISPLAY_TIMER, SPACE);
    return;
  }
}
